#!/usr/bin/env sh

# options
DESTDIR="/usr/local/bin/"
ORIGBIN="$PWD/bin/ncvt"
PROJDIR="ncvt"
MANPATH="/usr/share/man/man8"
ZIPPROG="gzip"
ROOTCMD="sudo" # default is sudo (because it's more widely used), doas is
# recommended though: it's safer and more minimalist
MKWICMD="makewhatis"
MKWIDIR="/usr/share/man"

# vars
success=0
ops=0

addSuccess() {
   success=$((success + 1))
}
addOp() {
   ops=$((ops + 1))
}

# check dir
[ "$(basename "$PWD")" != "$PROJDIR" ] && printf "Please make sure that you're in the project directory named \"%b\".\n" "$PROJDIR" && exit 1

# check root
touch /ncvtInstallationRootCheckFile 2>/dev/null &&
   rm /ncvtInstallationRootCheckFile 2>/dev/null &&
   printf "Running as root.\n" &&
   ROOTCMD=""

# compile
addOp
make &&
   printf "Compiled ncvt.\n" &&
   addSuccess

# copy the binary
addOp
$ROOTCMD mv "$ORIGBIN" "$DESTDIR" &&
   printf "Copied binary file over to \"%b\".\n" "$DESTDIR" &&
   addSuccess

# copy manpage
addOp
$ROOTCMD cp ncvt.1 "$MANPATH" &&
   printf "Copied manpage over to \"%b\".\n" "$MANPATH" &&
   addSuccess &&
   addOp &&
   $ROOTCMD $ZIPPROG "$MANPATH/ncvt.1" &&
   printf "%b'd manpage.\n" "$ZIPPROG" &&
   addSuccess

# run makewhatis
command -v makewhatis && {
   addOp
   $ROOTCMD $MKWICMD "$MKWIDIR" &&
      printf "Ran %b on %b.\n" "$MKWICMD" "$MKWIDIR" &&
      addSuccess
}

# done
if [ "$success" = "$ops" ]; then
   printf "Done! Enjoy.\n"
else
   printf "Error: installation failed.\n" && exit 1
fi
