//    Copyright (C) 2022-2023 mirk0dex <mirkodi.eu>
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// lib for ncvt: nUMBER cONvERtER

#ifndef NCVT_H_
#define NCVT_H_

#include "calcdigs.h" // calcDigits()
#include "strutils.h"
#include <math.h> // pow()
#include <stdio.h>
#include <stdlib.h> // exit()
#include <string.h> // strcpy(), strlen()

// vars
#define UNARYDIGIT "1"
#define MINUS '-'
#define EMPTYVALUE "0" // result to print instead of ""
#define CHARN 512      // max length of arrays

// convert number to base 10
char *anyToTen(char num[], int base) {
  int result = atoi(num);

  if (base != 10) {
    result = 0; // delete original num (we have it saved in strNum)
    for (unsigned int i = 0; i < strlen(num); ++i)
      if (num[i] - 48 < 10)
        result += (num[i] - 48 /* ASCII */) *
                  pow(base, strlen(num) - (i + 1)); // numbers
      else
        result += (num[i] - 55 /* ASCII */) *
                  pow(base, strlen(num) - (i + 1)); // other
                                                    // chars
  }

  return itoa(result);
}

// convert denary number to any base
char *tenToAny(char num[], int targBase) {
  char *finNum = malloc(CHARN);
  int rem = 0, digits = 0, intNum = atoi(num);

  // convert number to targBase
  while (intNum > 0) {
    rem = intNum % targBase; // calculate remainder

    if (targBase <= 10)
      rem /= pow(10, calcDigits(rem) - 1); // truncate remainder to its
                                           // first digit

    if (rem >= 10)
      finNum[digits] = 55 + rem; // save obtained remainder (48
    else                         // is 0; 65 is A)
      finNum[digits] = 48 + rem; // same, but using letters

    intNum /= targBase; // update num, too
    ++digits;           // increase iteration counter
  }

  finNum = flip(finNum);

  // make sure "" doesn't get returned if result is 0
  if (strcmp(finNum, "") == 0)
    strcpy(finNum, EMPTYVALUE);

  return finNum;
}

// convert base 1 number to base 10
char *unaryToTen(char num[]) { return itoa(strlen(num)); }

// convert base 10 number to base 1
char *tenToUnary(char num[]) {
  char *finNum = malloc(CHARN);
  for (int i = 0; i < atoi(num); ++i)
    strcat(finNum, UNARYDIGIT);

  return finNum;
}

// convert number to target base
char *convert(char num[], int base, int targBase) {
  int negative = 0; // remember to add minus
  if (num[0] == MINUS) {
    negative = 1;
    num += 1; // temporarily remove minus (first char)
  }

  char *numBase10 = malloc(CHARN);
  if (base == 1)
    numBase10 = unaryToTen(num);
  else
    numBase10 = anyToTen(num, base);

  char *numBaseX = malloc(CHARN);
  if (targBase == 1)
    numBaseX = tenToUnary(numBase10);
  else
    numBaseX = tenToAny(numBase10, targBase);

  // re-add minus if needed
  char *finNum = malloc(strlen(numBaseX) + 2);
  if (negative) {
    for (unsigned long i = 0; i < strlen(numBaseX); ++i)
      finNum[i + 1] = numBaseX[i];
    finNum[0] = MINUS;
  } else {
    strcpy(finNum, numBaseX);
  }

  return finNum;
}

#endif // NCVT_H_
