//    Copyright (C) 2022-2023 mirk0dex <mirkodi.eu>
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <https://www.gnu.org/licenses/>.

// ncvt: nUMBER cONvERtER

#include "lib/ncvt.h"
#include <stdio.h>

#define ERRORPREMSG "Error:"
#define ERRORCODE 1
#define BASELIMIT 127 // dictated by ASCII limits

// print error and exit
void error(char msg[], int errorCode) {
  printf("%s %s.\n", ERRORPREMSG, msg);
  exit(errorCode);
}

// print help and exit
void help(int errorCode) {
  printf("Usage: ncvt [NUMBER TO CONVERT] [INITIAL BASE] [TARGET "
         "BASE]\nConvert numbers between bases.\n");
  exit(errorCode);
}

void checks(int errorCode, char num[], int base, int targBase) {
  int asciiSub = 48;

  if (base == 0 || targBase == 0)
    error("base 0 doesn't exist", errorCode);
  else if (base > BASELIMIT || targBase > BASELIMIT) {
    char msg[CHARN];
    sprintf(msg, "bases can't exceed %d", BASELIMIT);
    error(msg, errorCode);
  }

  if (base != 1) {
    for (unsigned int i = 0; i < strlen(num); ++i) {
      if (num[i] - asciiSub > 9)
        asciiSub = 55;
      if (num[i] - asciiSub >= base) {
        char msg[CHARN];
        sprintf(msg, "illegal character \"%c\" does not exists in base-%d",
                num[i], base);
        error(msg, errorCode);
      }
    }
  }
}

int main(int argc, char **argv) {
  if (argc != 4)
    help(1); // if exactly 3 args aren't given, show help and quit (w/ error)

  int base = atoi(argv[2]), targBase = atoi(argv[3]);

  checks(ERRORCODE, argv[1], base, targBase);

  printf("%s\n", convert(argv[1], base, targBase));

  return 0;
}
